%global __tar %{_bindir}/tar

%define python3_sitearch %(python3 -c "from distutils.sysconfig import get_python_lib; print(get_python_lib(1))")

Name:           faiss 
Version:        1.9.0 
Release:        2 
Summary:        A library for efficient similarity search and clustering of dense vectors. 

License:        MIT 
URL:            https://github.com/facebookresearch/faiss 
Source:         https://github.com/facebookresearch/faiss/archive/refs/tags/faiss-%{version}.tar.gz 

BuildArch:      ppc

BuildRequires:  gcc >= 10.3.0 
BuildRequires:  cmake >= 3.30.3
BuildRequires:  openblas >= 0.3.18
BuildRequires:  gflags >= 2.2.2
BuildRequires:  swig >= 4.1.1
BuildRequires:  libgomp >= 10.3.0

Requires:  openblas >= 0.3.18
Requires:  libgomp >= 10.3.0

%description
Faiss is a library for efficient similarity search and clustering of dense vectors. It contains algorithms that search in sets of vectors of any size, up to ones that possibly do not fit in RAM. It also contains supporting code for evaluation and parameter tuning. Faiss is written in C++ with complete wrappers for Python/numpy. Some of the most useful algorithms are implemented on the GPU. It is developed primarily at Meta's Fundamental AI Research group.

%package devel
Summary: Devel package for faiss 

Requires: %{name} >= %{version}-%{release}

%description devel
Devel package for faiss.

%package python3 
Summary: python3 package for faiss

BuildRequires:  python3.9 >= 3.9.18
BuildRequires:  python3.9-numpy >= 2.0.2
Requires: %{name} >= %{version}-%{release}
Requires:  python3.9 >= 3.9.18
Requires:  python3.9-numpy >= 2.0.2

%description python3 
python3 package for faiss.

%prep
%setup -q

%build
export CC="%{__cc__}"
export CXX="%{__cxx__}"
export CXXFLAGS=" -maix64 -DFINTEGER=long -fopenmp -I/opt/freeware/include -I. "
export CFLAGS=$CXXFLAGS
export LDFLAGS="-lpython3.9 -L/opt/freeware/lib/pthread/ppc64/  -L/opt/freeware/lib64 -L/opt/freeware/lib -lpthread -Wl,-blibpath:/opt/freeware/lib64:/opt/freeware/lib:/usr/lib:/lib"
export OBJECT_MODE=64
export VERBOSE=1

cmake . -B build \
  -DFAISS_ENABLE_GPU=OFF \
  -DBLA_VENDOR=OpenBLAS \
  -DBLAS_LIBRARIES=/opt/freeware/lib/libopenblas.a \
  -DLAPACK_LIBRARIES=/opt/freeware/lib/libopenblas.a \
  -DFAISS_ENABLE_PYTHON=ON \
  -DBUILD_TESTING=OFF \
  -DBUILD_SHARED_LIBS=ON \
  -DCMAKE_BUILD_TYPE=Release \
  -DCMAKE_INSTALL_PREFIX=${PWD}/install_dir \
  -DCMAKE_AIX_SHARED_LIBRARY_ARCHIVE=ON \
  -DCMAKE_INSTALL_PREFIX=/opt/freeware \
  -DCMAKE_INSTALL_LIBDIR=lib64

cmake --build build -j 4

%install
export DESTDIR=${RPM_BUILD_ROOT}
export OBJECT_MODE=64
cmake --install build

cd build/faiss/python && python3.9 setup.py install --root=${RPM_BUILD_ROOT}

# As of now setup.py does not have support to add python files in lib64 as well as libfaiss_python_callbacks.a.
# These need to be worked with the community for future versions. These two commands below need to be
# removed in future versions if setup.py is set right.

cp libfaiss_python_callbacks.a ${RPM_BUILD_ROOT}/%{_libdir}64/
cp -r ${RPM_BUILD_ROOT}/%{_libdir}/* ${RPM_BUILD_ROOT}/%{_libdir}64/ 

%clean
[ "${RPM_BUILD_ROOT}" != "/" ] && rm -rf ${RPM_BUILD_ROOT}

%files
%defattr(-,root,system,-)
%license LICENSE 
%{_libdir}64/*.a
%exclude %{_libdir}64/libfaiss_python_callbacks.a

%files devel
%defattr(-,root,system,-)
%{_includedir}/faiss/*

%files python3
%defattr(-,root,system,-)
%{python3_sitearch}/faiss/*
%{_libdir}64/libfaiss_python_callbacks.a

%changelog
* Mon Dec 02 2024 Aditya Kamath <Aditya.Kamath1@ibm.com> - 1.9.0-2
- Rebuild with numpy >= 2

* Wed Oct 09 2024 Aditya Kamath <Aditya.Kamath1@ibm.com> - 1.9.0-1
- Port FAISS to AIX.
